#ifndef PARSER_H
#define PARSER_H

#include <exception>
#include "lex.hpp"
#include "symbol-table.hpp"

class Parser {
   public:
      class Exception: public std::exception {
	 public:
	    Exception(const Token& token, const std::string& _msg);
	    virtual ~Exception() noexcept;
	    virtual const char* what() const noexcept;
	 private:
	    std::string msg;
      };

      Parser(Lex& _lex, SymbolTable& _symtab);

      ExpressionPtr getExpr() throw(Exception);

   private:
      Lex& lex; // input channel
      SymbolTable& symtab; // used for macros

      Token token; // current token
      bool tokenConsumed; // true, if we need to fetch the next token

      void nextToken();
      Token getToken();

      ExpressionPtr parseAssignment() throw(Exception);
      ExpressionPtr parseSimpleExpression() throw(Exception);
      ExpressionPtr parseTerm() throw(Exception);
      ExpressionPtr parseFactor() throw(Exception);
};

#endif
