#ifndef SYMBOL_TABLE_H
#define SYMBOL_TABLE_H

#include <map>
#include <string>
#include "types.hpp"
#include "expression.hpp"

class SymbolTable {
   public:
      /* define or redefine a macro */
      void define(const std::string& name, ExpressionPtr expr) {
	 table[name] = expr;
      }

      /* retrieve the expression associated with
	 the given name; null is returned if the name
	 has not been bound to an expression */
      ExpressionPtr get(const std::string& name) const {
	 auto it = table.find(name);
	 if (it == table.end()) {
	    return ExpressionPtr((Expression*) nullptr);
	 }
	 return it->second;
      }

   private:
      std::map<std::string, ExpressionPtr> table;
};

#endif
